package com.compomics.marc;

import com.compomics.util.math.statistics.ROC;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;

/**
 *
 * @author Marc
 */
public class GeneMapping {

    private static final String separator = "\t";

    private HashMap<String, HashMap<String, int[]>> geneStartEnd;

    private HashMap<String, Integer> chromosomeLengths = new HashMap<String, Integer>();

    private HashMap<String, int[]> peptideAtlasNPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> maxqdbNPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> gpmdbNPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride0NPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride50NPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride75NPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride85NPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride90NPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride95NPeptides = new HashMap<String, int[]>();
    private HashMap<String, int[]> pride99NPeptides = new HashMap<String, int[]>();

    public static void main(String[] args) {
        GeneMapping m = new GeneMapping();

        try {

            m.mapGenes();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void mapGenes() throws FileNotFoundException, IOException {

        System.out.println("Importing gene information");
        importGeneProperties();

        System.out.println(new Date() + " Importing PeptideAtlas");
        getPeptideAtlasMapping();
        System.out.println(new Date() + " Importing MaxqDB");
        getMaxQDBMapping();
        System.out.println(new Date() + " Importing GPMDB");
        getGpmDbMapping();
        System.out.println(new Date() + " Importing Respin");
        getRespinMapping();
        System.out.println(new Date() + " Exporting Results");
        exportResults();

    }

    private void importGeneProperties() throws FileNotFoundException, IOException {

        geneStartEnd = new HashMap<String, HashMap<String, int[]>>();

        File inFile = new File("D:\\projects\\human mapping\\mart_export.txt");
        BufferedReader br = new BufferedReader(new FileReader(inFile));

        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {

                String[] split = line.split("\t");

                String geneId = split[0];
                int start = new Integer(split[1]);
                int end = new Integer(split[2]);
                int[] startEnd = {start, end};
                String chromosome = split[3];

                HashMap<String, int[]> geneMapping = geneStartEnd.get(geneId);
                if (geneMapping == null) {
                    geneMapping = new HashMap<String, int[]>(100);
                    geneStartEnd.put(geneId, geneMapping);
                }
                geneMapping.put(chromosome, startEnd);

                Integer chromosomeLength = chromosomeLengths.get(chromosome);
                if (chromosomeLength == null || chromosomeLength < end) {
                    chromosomeLengths.put(chromosome, end);
                }

            }

        } finally {
            br.close();
        }

    }

    HashMap<String, ArrayList<String>> getPeptideToGeneMapping(String fileName) throws FileNotFoundException, IOException {

        File inFile = new File("D:\\projects\\human mapping\\mapped_peptides", fileName);

        HashMap<String, ArrayList<String>> result = new HashMap<String, ArrayList<String>>(100);
        BufferedReader br = new BufferedReader(new FileReader(inFile));

        try {
            String line;
            while ((line = br.readLine()) != null) {
                String[] split = line.split("\t");
                String peptide = split[0].toUpperCase();
                if (split.length > 2) {
                    ArrayList<String> peptideGenes = result.get(peptide);
                    if (peptideGenes == null) {
                        peptideGenes = new ArrayList<String>();
                        result.put(peptide, peptideGenes);
                    }
                    String genes = split[2];
                    split = genes.split(";");
                    for (String mapping : split) {
                        String gene = mapping.split(",")[0];
                        peptideGenes.add(gene);
                    }
                }
            }
        } finally {
            br.close();
        }

        return result;

    }

    private void getPeptideAtlasMapping() throws IOException {

        HashMap<String, ArrayList<String>> peptideMapping = getPeptideToGeneMapping("peptideatlas_mapped.txt");

        File inFile = new File("D:\\projects\\human mapping\\origin_peptide_lists", "peptideatlas_9606.tsv");

        BufferedReader br = new BufferedReader(new FileReader(inFile));

        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] split = line.split("\t");
                String peptide = split[1].toUpperCase();
                Double bestP = new Double(split[2]);

                ArrayList<String> genes = peptideMapping.get(peptide);

                if (genes != null) {
                    for (String gene : genes) {
                        HashMap<String, int[]> mapping = geneStartEnd.get(gene);
                        if (mapping == null) {
                            throw new IllegalArgumentException("Gene " + gene + " not mapped.");
                        }
                        for (String chromosome : mapping.keySet()) {

                            int chromosomeLength = chromosomeLengths.get(chromosome);

                            int[] startEnd = mapping.get(chromosome);

                            int[] peptideCoverage = peptideAtlasNPeptides.get(chromosome);
                            if (peptideCoverage == null) {
                                peptideCoverage = new int[100];
                                peptideAtlasNPeptides.put(chromosome, peptideCoverage);
                            }

                            int lastCoverage = -1;
                            for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                double position = 100 * bp / chromosomeLength;
                                int percent = (int) position;
                                if (percent != lastCoverage) {
                                    lastCoverage = percent;
                                    peptideCoverage[percent]++;
                                }
                            }
                        }
                    }
                }

            }
        } finally {
            br.close();
        }

    }

    private void getMaxQDBMapping() throws IOException {

        HashMap<String, ArrayList<String>> peptideMapping = getPeptideToGeneMapping("maxqdb_mapped.txt");

        File inFile = new File("D:\\projects\\human mapping\\origin_peptide_lists", "maxqb_9606.tsv");

        BufferedReader br = new BufferedReader(new FileReader(inFile));

        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] split = line.split("\t");
                String peptide = split[0].toUpperCase();
                Double bestP = new Double(split[1]);
                bestP = 1 - bestP;

                ArrayList<String> genes = peptideMapping.get(peptide);

                if (genes != null) {
                    for (String gene : genes) {
                        HashMap<String, int[]> mapping = geneStartEnd.get(gene);
                        if (mapping == null) {
                            throw new IllegalArgumentException("Gene " + gene + " not mapped.");
                        }
                        for (String chromosome : mapping.keySet()) {
                            int chromosomeLength = chromosomeLengths.get(chromosome);

                            int[] startEnd = mapping.get(chromosome);

                            int[] peptideCoverage = maxqdbNPeptides.get(chromosome);
                            if (peptideCoverage == null) {
                                peptideCoverage = new int[100];
                                maxqdbNPeptides.put(chromosome, peptideCoverage);
                            }

                            int lastCoverage = -1;
                            for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                double position = 100 * bp / chromosomeLength;
                                int percent = (int) position;
                                if (percent != lastCoverage) {
                                    lastCoverage = percent;
                                    peptideCoverage[percent]++;
                                }
                            }
                        }
                    }
                }

            }
        } finally {
            br.close();
        }

    }

    private void getGpmDbMapping() throws IOException {

        HashMap<String, ArrayList<String>> peptideMapping = getPeptideToGeneMapping("gpmdb_mapped.txt");

        File inFile = new File("D:\\projects\\human mapping\\origin_peptide_lists", "gpmdb_9606.tsv");

        BufferedReader br = new BufferedReader(new FileReader(inFile));

        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] split = line.split("\t");
                String peptide = split[0].toUpperCase();
                Double bestP = new Double(split[5]);
                bestP = 1 - bestP;

                ArrayList<String> genes = peptideMapping.get(peptide);

                if (genes != null) {
                    for (String gene : genes) {
                        HashMap<String, int[]> mapping = geneStartEnd.get(gene);
                        if (mapping == null) {
                            throw new IllegalArgumentException("Gene " + gene + " not mapped.");
                        }
                        for (String chromosome : mapping.keySet()) {
                            int chromosomeLength = chromosomeLengths.get(chromosome);

                            int[] startEnd = mapping.get(chromosome);

                            int[] peptideCoverage = gpmdbNPeptides.get(chromosome);
                            if (peptideCoverage == null) {
                                peptideCoverage = new int[100];
                                gpmdbNPeptides.put(chromosome, peptideCoverage);
                            }

                            int lastCoverage = -1;
                            for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                double position = 100 * bp / chromosomeLength;
                                int percent = (int) position;
                                if (percent != lastCoverage) {
                                    lastCoverage = percent;
                                    peptideCoverage[percent]++;
                                }
                            }
                        }
                    }
                }

            }
        } finally {
            br.close();
        }

    }

    private void getRespinMapping() throws IOException {

        HashMap<String, ArrayList<String>> peptideMapping = getPeptideToGeneMapping("respin_mapped.txt");

        File inFile = new File("D:\\projects\\human mapping\\origin_peptide_lists", "ReSpin_peptide_list_PEP.csv");

        BufferedReader br = new BufferedReader(new FileReader(inFile));

        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] split = line.split("\t");
                String peptide = split[0].toUpperCase();
                Double bestP = new Double(split[4]);
                bestP = (100 - bestP) / 100;

                ArrayList<String> genes = peptideMapping.get(peptide);

                if (genes != null) {
                    for (String gene : genes) {
                        HashMap<String, int[]> mapping = geneStartEnd.get(gene);
                        if (mapping == null) {
                            throw new IllegalArgumentException("Gene " + gene + " not mapped.");
                        }
                        for (String chromosome : mapping.keySet()) {
                            int chromosomeLength = chromosomeLengths.get(chromosome);
                            int[] startEnd = mapping.get(chromosome);

                            if (bestP > 0.99) {

                                int[] peptideCoverage = pride99NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride99NPeptides.put(chromosome, peptideCoverage);
                                }
                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            } else if (bestP > 0.95) {

                                int[] peptideCoverage = pride95NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride95NPeptides.put(chromosome, peptideCoverage);
                                }

                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            } else if (bestP > 0.90) {

                                int[] peptideCoverage = pride90NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride90NPeptides.put(chromosome, peptideCoverage);
                                }

                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            } else if (bestP > 0.85) {

                                int[] peptideCoverage = pride85NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride85NPeptides.put(chromosome, peptideCoverage);
                                }

                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            } else if (bestP > 0.75) {

                                int[] peptideCoverage = pride75NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride75NPeptides.put(chromosome, peptideCoverage);
                                }

                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            } else if (bestP > 0.50) {

                                int[] peptideCoverage = pride50NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride50NPeptides.put(chromosome, peptideCoverage);
                                }

                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            } else {

                                int[] peptideCoverage = pride0NPeptides.get(chromosome);
                                if (peptideCoverage == null) {
                                    peptideCoverage = new int[100];
                                    pride0NPeptides.put(chromosome, peptideCoverage);
                                }

                                int lastCoverage = -1;
                                for (double bp = startEnd[0]; bp < startEnd[1]; bp++) {
                                    double position = 100 * bp / chromosomeLength;
                                    int percent = (int) position;
                                    if (percent != lastCoverage) {
                                        lastCoverage = percent;
                                        peptideCoverage[percent]++;
                                    }
                                }
                            }
                        }
                    }
                }

            }
        } finally {
            br.close();
        }

    }

    private void exportResults() throws IOException {

        File outFile = new File("D:\\projects\\human mapping\\mapping.txt");
        BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
        ArrayList<String> chromosomes = new ArrayList<String>(chromosomeLengths.keySet());
        Collections.sort(chromosomes);
        try {
            for (String chromosome : chromosomes) {
                
                bw.newLine();
                int chromosomeLength = chromosomeLengths.get(chromosome);
                System.out.println("Chromosome " + chromosome + "(" + chromosomeLength + " bp)");
                bw.write("Chromosome" + separator + chromosome + "(" + chromosomeLength + " bp)");
                bw.newLine();
                bw.write("bp [%]");
                for (int bp = 0; bp < 100; bp++) {
                    bw.write(separator + bp);
                }
                bw.newLine();
                bw.write("PeptideAtlas");
                int[] chromosomeMapping = peptideAtlasNPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("MaxqDB");
                chromosomeMapping = maxqdbNPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("GPMDB");
                chromosomeMapping = gpmdbNPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 0% - 50%");
                chromosomeMapping = pride0NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 50% - 75%");
                chromosomeMapping = pride50NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 75% - 85%");
                chromosomeMapping = pride75NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 85% - 90%");
                chromosomeMapping = pride85NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 90% - 95%");
                chromosomeMapping = pride90NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 95% - 99%");
                chromosomeMapping = pride95NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
                bw.write("PRIDE 99% - 100%");
                chromosomeMapping = pride99NPeptides.get(chromosome);
                for (int i = 0; i < 100; i++) {
                    if (chromosomeMapping != null) {
                        bw.write(separator + chromosomeMapping[i]);
                    } else {
                        bw.write(separator + 0.0);
                    }
                }
                bw.newLine();
            }
            bw.newLine();

        } finally {
            bw.close();
        }
    }

}
